@@ -46,16 +46,19 @@
 #include "dkp-history-obj.h"
 #include "dkp-stats-obj.h"
 #include "dkp-device.h"
+#include "dkp-wakeups.h"
 
 static GladeXML *glade_xml = NULL;
 static GtkListStore *list_store_info = NULL;
 static GtkListStore *list_store_devices = NULL;
+static GtkListStore *list_store_wakeups = NULL;
 gchar *current_device = NULL;
 static const gchar *history_type;
 static const gchar *stats_type;
 static guint history_time;
 static GConfClient *gconf_client;
-static EggArrayFloat *gaussian = NULL;
+static gfloat sigma_smoothing = 0.0f;
+static DkpWakeups *wakeups = NULL;
 
 enum {
 	GPM_INFO_COLUMN_TEXT,
@@ -70,6 +73,15 @@ enum {
 	GPM_DEVICES_COLUMN_LAST
 };
 
+enum {
+	GPM_WAKEUPS_COLUMN_ICON,
+	GPM_WAKEUPS_COLUMN_ID,
+	GPM_WAKEUPS_COLUMN_VALUE,
+	GPM_WAKEUPS_COLUMN_CMDLINE,
+	GPM_WAKEUPS_COLUMN_DETAILS,
+	GPM_WAKEUPS_COLUMN_LAST
+};
+
 #define GPM_HISTORY_RATE_TEXT			_("Rate")
 #define GPM_HISTORY_CHARGE_TEXT			_("Charge")
 #define GPM_HISTORY_TIME_FULL_TEXT		_("Time to full")
@@ -157,6 +169,51 @@ gpm_stats_add_devices_columns (GtkTreeView *treeview)
 	gtk_tree_view_column_set_expand (column, TRUE);
 }
 
+/**
+ * gpm_stats_add_wakeups_columns:
+ **/
+static void
+gpm_stats_add_wakeups_columns (GtkTreeView *treeview)
+{
+	GtkCellRenderer *renderer;
+	GtkTreeViewColumn *column;
+
+	/* image */
+	renderer = gtk_cell_renderer_pixbuf_new ();
+	g_object_set (renderer, "stock-size", GTK_ICON_SIZE_BUTTON, NULL);
+	column = gtk_tree_view_column_new_with_attributes (_("Type"), renderer,
+							   "icon-name", GPM_WAKEUPS_COLUMN_ICON, NULL);
+	gtk_tree_view_append_column (treeview, column);
+
+	/* column for id */
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes (_("ID"), renderer,
+							   "markup", GPM_WAKEUPS_COLUMN_ID, NULL);
+	gtk_tree_view_append_column (treeview, column);
+	gtk_tree_view_column_set_expand (column, TRUE);
+
+	/* column for value */
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes (_("Wakeups"), renderer,
+							   "markup", GPM_WAKEUPS_COLUMN_VALUE, NULL);
+	gtk_tree_view_append_column (treeview, column);
+	gtk_tree_view_column_set_expand (column, TRUE);
+
+	/* column for cmdline */
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes (_("Command"), renderer,
+							   "markup", GPM_WAKEUPS_COLUMN_CMDLINE, NULL);
+	gtk_tree_view_append_column (treeview, column);
+	gtk_tree_view_column_set_expand (column, TRUE);
+
+	/* column for details */
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes (_("Details"), renderer,
+							   "markup", GPM_WAKEUPS_COLUMN_DETAILS, NULL);
+	gtk_tree_view_append_column (treeview, column);
+	gtk_tree_view_column_set_expand (column, TRUE);
+}
+
 /**
  * gpm_stats_add_info_data:
  **/
@@ -182,6 +239,8 @@ gpm_stats_update_smooth_data (EggObjList *list)
 	EggObjList *new;
 	EggArrayFloat *raw;
 	EggArrayFloat *convolved;
+	EggArrayFloat *outliers;
+	EggArrayFloat *gaussian = NULL;
 
 	/* convert the y data to a EggArrayFloat array */
 	raw = egg_array_float_new (list->len);
@@ -191,8 +250,12 @@ gpm_stats_update_smooth_data (EggObjList *list)
 		egg_array_float_set (raw, i, point->y);
 	}
 
+	/* remove any outliers */
+	outliers = egg_array_float_remove_outliers (raw, 3, 0.1);
+
 	/* convolve with gaussian */
-	convolved = egg_array_float_convolve (raw, gaussian);
+	gaussian = egg_array_float_compute_gaussian (15, sigma_smoothing);
+	convolved = egg_array_float_convolve (outliers, gaussian);
 
 	/* add the smoothed data back into a new array */
 	new = egg_obj_list_new ();
@@ -207,8 +270,10 @@ gpm_stats_update_smooth_data (EggObjList *list)
 	}
 
 	/* free data */
+	egg_array_float_free (gaussian);
 	egg_array_float_free (raw);
 	egg_array_float_free (convolved);
+	egg_array_float_free (outliers);
 
 	return new;
 }
@@ -443,6 +508,8 @@ gpm_stats_update_info_page_history (const DkpDevice *device)
 		dkp_point_obj_free (point);
 	}
 
+	/* render */
+	sigma_smoothing = 2.0;
 	widget = glade_xml_get_widget (glade_xml, "checkbutton_smooth_history");
 	checked = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
 	widget = glade_xml_get_widget (glade_xml, "checkbutton_points_history");
@@ -522,6 +589,7 @@ gpm_stats_update_info_page_stats (const DkpDevice *device)
 	}
 
 	/* render */
+	sigma_smoothing = 1.1;
 	widget = glade_xml_get_widget (glade_xml, "checkbutton_smooth_stats");
 	checked = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
 	widget = glade_xml_get_widget (glade_xml, "checkbutton_points_stats");
@@ -565,6 +633,10 @@ gpm_stats_update_info_data (const DkpDevice *device)
 	widget = glade_xml_get_widget (glade_xml, "notebook1");
 	obj = dkp_device_get_object (device);
 
+	/* show info page */
+	page_widget = gtk_notebook_get_nth_page (GTK_NOTEBOOK(widget), 0);
+	gtk_widget_show (page_widget);
+
 	/* hide history if no support */
 	page_widget = gtk_notebook_get_nth_page (GTK_NOTEBOOK(widget), 1);
 	if (obj->has_history)
@@ -579,12 +651,201 @@ gpm_stats_update_info_data (const DkpDevice *device)
 	else
 		gtk_widget_hide (page_widget);
 
+	/* hide wakeups page */
+	page_widget = gtk_notebook_get_nth_page (GTK_NOTEBOOK(widget), 3);
+	gtk_widget_hide (page_widget);
+
 	page = gtk_notebook_get_current_page (GTK_NOTEBOOK (widget));
 	gpm_stats_update_info_data_page (device, page);
 
 	return;
 }
 
+/**
+ * gpm_stats_add_wakeups_obj:
+ **/
+static void
+gpm_stats_add_wakeups_obj (const DkpWakeupsObj *obj)
+{
+	const gchar *icon;
+	gchar *value;
+	gchar *cmdline;
+	gchar *id;
+	const gchar *cmdline_ptr;
+	gchar *cmdline_escaped;
+	gchar *details;
+	gchar *found;
+	GtkTreeIter iter;
+
+	if (obj->is_userspace) {
+		icon = "application-x-executable";
+		id = g_strdup_printf ("%i", obj->id);
+	} else {
+		icon = "applications-system";
+		if (obj->id < 0xff0)
+			id = g_strdup_printf ("IRQ%i", obj->id);
+		else
+			id = g_strdup ("IRQx");
+	}
+
+	/* formate value to one decimal place */
+	value = g_strdup_printf ("%.1f", obj->value);
+
+	/* truncate at first space or ':' */
+	cmdline = g_strdup (obj->cmdline);
+	found = strstr (cmdline, ":");
+	if (found != NULL)
+		*found = '\0';
+	found = strstr (cmdline, " ");
+	if (found != NULL)
+		*found = '\0';
+
+	/* remove ./ */
+	found = g_strrstr (cmdline, "/");
+	if (found != NULL)
+		cmdline_ptr = found + 1;
+	else
+		cmdline_ptr = cmdline;
+
+	if (strcmp (cmdline_ptr, "insmod") == 0)
+		cmdline_ptr = _("Kernel module");
+	else if (strcmp (cmdline_ptr, "modprobe") == 0)
+		cmdline_ptr = _("Kernel module");
+	else if (strcmp (cmdline_ptr, "swapper") == 0)
+		cmdline_ptr = _("Kernel core");
+	else if (strcmp (cmdline_ptr, "kernel-ipi") == 0)
+		cmdline_ptr = _("Interprocessor interrupt");
+	else if (strcmp (cmdline_ptr, "interrupt") == 0)
+		cmdline_ptr = _("Interrupt");
+
+	/* format command line */
+	if (obj->is_userspace)
+		cmdline_escaped = g_markup_escape_text (cmdline_ptr, -1);
+	else
+		cmdline_escaped = g_markup_printf_escaped ("<i>%s</i>", cmdline_ptr);
+
+	/* replace common driver names */
+	if (strcmp (obj->details, "i8042") == 0)
+		details = g_strdup (_("PS/2 keyboard/mouse/touchpad"));
+	else if (strcmp (obj->details, "acpi") == 0)
+		details = g_strdup (_("ACPI"));
+	else if (strcmp (obj->details, "ata_piix") == 0)
+		details = g_strdup (_("Serial ATA"));
+	else if (strcmp (obj->details, "libata") == 0)
+		details = g_strdup (_("ATA host controller"));
+	else if (strcmp (obj->details, "iwl3945") == 0)
+		details = g_strdup (_("Intel wireless adaptor"));
+
+	/* try to make the wakeup type nicer */
+	else if (g_str_has_prefix (obj->details, "__mod_timer"))
+		details = g_strdup_printf (_("Timer %s"), obj->details+12);
+	else if (g_str_has_prefix (obj->details, "mod_timer"))
+		details = g_strdup_printf (_("Timer %s"), obj->details+10);
+	else if (g_str_has_prefix (obj->details, "do_setitimer"))
+		details = g_strdup_printf (_("Timer %s"), obj->details+10);
+	else if (g_str_has_prefix (obj->details, "do_nanosleep"))
+		details = g_strdup_printf (_("Sleep %s"), obj->details+13);
+	else if (g_str_has_prefix (obj->details, "futex_wait"))
+		details = g_strdup_printf (_("Wait %s"), obj->details+11);
+	else if (g_str_has_prefix (obj->details, "queue_delayed_work_on"))
+		details = g_strdup_printf (_("Work queue %s"), obj->details+22);
+	else if (g_str_has_prefix (obj->details, "queue_delayed_work"))
+		details = g_strdup_printf (_("Work queue %s"), obj->details+19);
+	else if (g_str_has_prefix (obj->details, "dst_run_gc"))
+		details = g_strdup_printf (_("Network route flush %s"), obj->details+11);
+	else if (g_str_has_prefix (obj->details, "usb_hcd_poll_rh_status"))
+		details = g_strdup_printf (_("USB activity %s"), obj->details+23);
+	else if (g_str_has_prefix (obj->details, "Local timer interrupts"))
+		details = g_strdup (_("Local interrupts"));
+	else if (g_str_has_prefix (obj->details, "Rescheduling interrupts"))
+		details = g_strdup (_("Rescheduling interrupts"));
+	else
+		details = g_markup_escape_text (obj->details, -1);
+
+	gtk_list_store_append (list_store_wakeups, &iter);
+	gtk_list_store_set (list_store_wakeups, &iter,
+			    GPM_WAKEUPS_COLUMN_ID, id,
+			    GPM_WAKEUPS_COLUMN_VALUE, value,
+			    GPM_WAKEUPS_COLUMN_CMDLINE, cmdline_escaped,
+			    GPM_WAKEUPS_COLUMN_DETAILS, details,
+			    GPM_WAKEUPS_COLUMN_ICON, icon, -1);
+	g_free (cmdline);
+	g_free (cmdline_escaped);
+	g_free (details);
+	g_free (value);
+	g_free (id);
+}
+
+/**
+ * gpm_stats_update_wakeups_data:
+ **/
+static void
+gpm_stats_update_wakeups_data (void)
+{
+	GtkWidget *widget;
+	GtkWidget *page_widget;
+	guint total;
+	DkpWakeupsObj *obj;
+	gchar *text;
+	guint i;
+	GError *error = NULL;
+	GPtrArray *array;
+
+	widget = glade_xml_get_widget (glade_xml, "notebook1");
+
+	/* hide other pages */
+	page_widget = gtk_notebook_get_nth_page (GTK_NOTEBOOK(widget), 0);
+	gtk_widget_hide (page_widget);
+	page_widget = gtk_notebook_get_nth_page (GTK_NOTEBOOK(widget), 1);
+	gtk_widget_hide (page_widget);
+	page_widget = gtk_notebook_get_nth_page (GTK_NOTEBOOK(widget), 2);
+	gtk_widget_hide (page_widget);
+
+	/* show wakeups page */
+	page_widget = gtk_notebook_get_nth_page (GTK_NOTEBOOK(widget), 3);
+	gtk_widget_show (page_widget);
+
+	/* show total */
+	total = dkp_wakeups_get_total (wakeups, &error);
+	widget = glade_xml_get_widget (glade_xml, "label_total_wakeups");
+	if (error == NULL) {
+		text = g_strdup_printf ("%i", total);
+		gtk_label_set_label (GTK_LABEL(widget), text);
+		g_free (text);
+	} else {
+		gtk_label_set_label (GTK_LABEL(widget), error->message);
+		g_error_free (error);
+	}
+
+	/* get data */
+	gtk_list_store_clear (list_store_wakeups);
+	array = dkp_wakeups_get_data (wakeups, NULL);
+	if (array == NULL)
+		return;
+	for (i=0; i<array->len; i++) {
+		obj = g_ptr_array_index (array, i);
+		gpm_stats_add_wakeups_obj (obj);
+	}
+	g_ptr_array_foreach (array, (GFunc) dkp_wakeups_obj_free, NULL);
+	g_ptr_array_free (array, TRUE);
+}
+
+static void
+gpm_stats_set_title (GtkWindow *window, gint page_num)
+{
+	gchar *title;
+	const gchar * const page_titles[] = {
+		N_("Device Information"),
+		N_("Device History"),
+		N_("Device Profile"),
+		N_("Processor Wakeups")
+	};
+
+	title = g_strdup_printf ("%s - %s", _("Power Statistics"), _(page_titles[page_num]));
+	gtk_window_set_title (window, title);
+	g_free (title);
+}
+
 /**
  * gpm_stats_notebook_changed_cb:
  **/
@@ -592,6 +853,11 @@ static void
 gpm_stats_notebook_changed_cb (GtkNotebook *notebook, GtkNotebookPage *page, gint page_num, gpointer user_data)
 {
 	DkpDevice *device;
+	GtkWidget *widget;
+
+	/* set the window title depending on the mode */
+	widget = glade_xml_get_widget (glade_xml, "dialog_stats");
+	gpm_stats_set_title (GTK_WINDOW (widget), page_num);
 
 	/* save page in gconf */
 	gconf_client_set_int (gconf_client, GPM_CONF_INFO_PAGE_NUMBER, page_num, NULL);
@@ -599,6 +865,9 @@ gpm_stats_notebook_changed_cb (GtkNotebook *notebook, GtkNotebookPage *page, gin
 	if (current_device == NULL)
 		return;
 
+	if (egg_strequal (current_device, "wakeups"))
+		return;
+
 	device = dkp_device_new ();
 	dkp_device_set_object_path (device, current_device);
 	gpm_stats_update_info_data_page (device, page_num);
@@ -641,6 +910,7 @@ gpm_stats_devices_treeview_clicked_cb (GtkTreeSelection *selection, gboolean dat
 {
 	GtkTreeModel *model;
 	GtkTreeIter iter;
+	DkpDevice *device;
 
 	/* This will only work in single or browse selection mode! */
 	if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
@@ -653,11 +923,15 @@ gpm_stats_devices_treeview_clicked_cb (GtkTreeSelection *selection, gboolean dat
 		/* show transaction_id */
 		egg_debug ("selected row is: %s", current_device);
 
-		DkpDevice *device;
-		device = dkp_device_new ();
-		dkp_device_set_object_path (device, current_device);
-		gpm_stats_update_info_data (device);
-		g_object_unref (device);
+		/* is special device */
+		if (egg_strequal (current_device, "wakeups")) {
+			gpm_stats_update_wakeups_data ();
+		} else {
+			device = dkp_device_new ();
+			dkp_device_set_object_path (device, current_device);
+			gpm_stats_update_info_data (device);
+			g_object_unref (device);
+		}
 
 	} else {
 		egg_debug ("no row selected");
@@ -715,6 +989,16 @@ gpm_stats_add_device (const DkpDevice *device)
 			    GPM_DEVICES_COLUMN_ICON, icon, -1);
 }
 
+/**
+ * gpm_stats_data_changed_cb:
+ **/
+static void
+gpm_stats_data_changed_cb (DkpClient *client, gpointer user_data)
+{
+	if (egg_strequal (current_device, "wakeups"))
+		gpm_stats_update_wakeups_data ();
+}
+
 /**
  * gpm_stats_device_added_cb:
  **/
@@ -971,6 +1255,7 @@ main (int argc, char *argv[])
 	gint page;
 	const gchar *object_path;
 	gboolean checked;
+	gchar *last_device;
 
 	const GOptionEntry options[] = {
 		{ "verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose,
@@ -990,7 +1275,7 @@ main (int argc, char *argv[])
 	g_type_init ();
 
 	context = g_option_context_new (NULL);
-	g_option_context_set_summary (context, _("Software Log Viewer"));
+	g_option_context_set_summary (context, _("Power Statistics"));
 	g_option_context_add_main_entries (context, options, NULL);
 	g_option_context_parse (context, &argc, &argv, NULL);
 	g_option_context_free (context);
@@ -1013,9 +1298,6 @@ main (int argc, char *argv[])
 	/* get data from gconf */
 	gconf_client = gconf_client_get_default ();
 
-	/* create the gaussian dataset */
-	gaussian = egg_array_float_compute_gaussian (35, 4.5);
-
 	/* add application specific icons to search path */
 	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
                                            GPM_DATA G_DIR_SEPARATOR_S "icons");
@@ -1025,9 +1307,8 @@ main (int argc, char *argv[])
 
 	glade_xml = glade_xml_new (GPM_DATA "/gpm-statistics.glade", NULL, NULL);
 	widget = glade_xml_get_widget (glade_xml, "dialog_stats");
-	gtk_window_set_icon_name (GTK_WINDOW (widget), "gtk-help");
-	gtk_widget_set_size_request (widget, 800, 500);
-	gtk_window_set_icon_name (GTK_WINDOW (widget), GPM_STOCK_APP_ICON);
+	gtk_window_set_default_size (GTK_WINDOW(widget), 800, 500);
+	gtk_window_set_default_icon_name (GPM_STOCK_APP_ICON);
 
 	/* Get the main window quit */
 	g_signal_connect_swapped (widget, "delete_event", G_CALLBACK (gtk_main_quit), NULL);
@@ -1078,6 +1359,8 @@ main (int argc, char *argv[])
 	list_store_info = gtk_list_store_new (GPM_INFO_COLUMN_LAST, G_TYPE_STRING, G_TYPE_STRING);
 	list_store_devices = gtk_list_store_new (GPM_DEVICES_COLUMN_LAST, G_TYPE_STRING,
 						 G_TYPE_STRING, G_TYPE_STRING);
+	list_store_wakeups = gtk_list_store_new (GPM_WAKEUPS_COLUMN_LAST, G_TYPE_STRING,
+						 G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
 
 	/* create transaction_id tree view */
 	widget = glade_xml_get_widget (glade_xml, "treeview_info");
@@ -1100,6 +1383,16 @@ main (int argc, char *argv[])
 	gpm_stats_add_devices_columns (GTK_TREE_VIEW (widget));
 	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (widget)); /* show */
 
+	/* create wakeups tree view */
+	widget = glade_xml_get_widget (glade_xml, "treeview_wakeups");
+	gtk_tree_view_set_model (GTK_TREE_VIEW (widget),
+				 GTK_TREE_MODEL (list_store_wakeups));
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
+
+	/* add columns to the tree view */
+	gpm_stats_add_wakeups_columns (GTK_TREE_VIEW (widget));
+	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (widget)); /* show */
+
 	history_type = gconf_client_get_string (gconf_client, GPM_CONF_INFO_HISTORY_TYPE, NULL);
 	history_time = gconf_client_get_int (gconf_client, GPM_CONF_INFO_HISTORY_TIME, NULL);
 	if (history_type == NULL)
@@ -1168,6 +1461,9 @@ main (int argc, char *argv[])
 	g_signal_connect (client, "device-removed", G_CALLBACK (gpm_stats_device_removed_cb), NULL);
 	g_signal_connect (client, "device-changed", G_CALLBACK (gpm_stats_device_changed_cb), NULL);
 
+	wakeups = dkp_wakeups_new ();
+	g_signal_connect (wakeups, "data-changed", G_CALLBACK (gpm_stats_data_changed_cb), NULL);
+
 	/* coldplug */
 	devices = dkp_client_enumerate_devices (client);
 	if (devices == NULL)
@@ -1181,9 +1477,19 @@ main (int argc, char *argv[])
 		}
 	}
 
-	gchar *last_device;
 	last_device = gconf_client_get_string (gconf_client, GPM_CONF_INFO_LAST_DEVICE, NULL);
 
+	/* can we get wakeup data? */
+	i = dkp_wakeups_get_total (wakeups, NULL);
+	if (i > 0) {
+		GtkTreeIter iter;
+		gtk_list_store_append (list_store_devices, &iter);
+		gtk_list_store_set (list_store_devices, &iter,
+				    GPM_DEVICES_COLUMN_ID, "wakeups",
+				    GPM_DEVICES_COLUMN_TEXT, _("Processor"),
+				    GPM_DEVICES_COLUMN_ICON, "computer", -1);
+	}
+
 	/* set the correct focus on the last device */
 	for (i=0; i < devices->len; i++) {
 		object_path = (const gchar *) g_ptr_array_index (devices, i);
@@ -1216,9 +1522,9 @@ main (int argc, char *argv[])
 	gtk_main ();
 
 out:
-	egg_array_float_free (gaussian);
 	g_object_unref (gconf_client);
 	g_object_unref (client);
+	g_object_unref (wakeups);
 	g_object_unref (glade_xml);
 	g_object_unref (list_store_info);
 unique_out:
