@@ -38,30 +38,19 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <gtk/gtkstatusicon.h>
 #include <gconf/gconf-client.h>
+#include <dkp-device.h>
 
-#ifdef HAVE_DK_POWER
- #include <dkp-device.h>
-#endif
-
-#ifdef HAVE_DK_POWER
- #include "gpm-devicekit.h"
- #include "gpm-engine.h"
-#endif
+#include "egg-debug.h"
 
+#include "gpm-devicekit.h"
+#include "gpm-engine.h"
 #include "gpm-control.h"
 #include "gpm-common.h"
-#include "egg-debug.h"
 #include "gpm-notify.h"
-#include "gpm-cell-array.h"
-#include "gpm-cell.h"
-#include "gpm-cell-unit.h"
 #include "gpm-stock-icons.h"
 #include "gpm-tray-icon.h"
 
-static void     gpm_tray_icon_class_init (GpmTrayIconClass *klass);
-static void     gpm_tray_icon_init       (GpmTrayIcon      *tray_icon);
 static void     gpm_tray_icon_finalize   (GObject	   *object);
 
 #define GPM_TRAY_ICON_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_TRAY_ICON, GpmTrayIconPrivate))
@@ -71,11 +60,7 @@ struct GpmTrayIconPrivate
 	GConfClient		*conf;
 	GpmControl		*control;
 	GpmNotify		*notify;
-#ifndef HAVE_DK_POWER
-	GpmEngineCollection	*collection;
-#else
 	GpmEngine		*engine;
-#endif
 	GtkStatusIcon		*status_icon;
 	gboolean		 is_visible;
 	gboolean		 show_suspend;
@@ -98,19 +83,6 @@ static guint	 signals [LAST_SIGNAL] = { 0 };
 
 G_DEFINE_TYPE (GpmTrayIcon, gpm_tray_icon, G_TYPE_OBJECT)
 
-#ifndef HAVE_DK_POWER
-/**
- * gpm_tray_icon_set_collection_data:
- **/
-gboolean
-gpm_tray_icon_set_collection_data (GpmTrayIcon *icon, GpmEngineCollection *collection)
-{
-	g_return_val_if_fail (GPM_IS_TRAY_ICON (icon), FALSE);
-	icon->priv->collection = collection;
-	return TRUE;
-}
-#endif
-
 /**
  * gpm_tray_icon_enable_suspend:
  * @icon: This TrayIcon class instance
@@ -220,24 +192,23 @@ gpm_tray_icon_show_info_cb (GtkMenuItem *item, gpointer data)
 	gchar *description = NULL;
 	GtkWidget *dialog;
 	GtkWidget *image;
-#ifndef HAVE_DK_POWER
-	GpmCell *cell;
-#else
 	const DkpObject	*obj;
 	const gchar *object_path;
-	DkpDevice	*device = NULL;
+	DkpDevice *device = NULL;
 	gboolean ret;
-#endif
 
-#ifndef HAVE_DK_POWER
-	cell = g_object_get_data (G_OBJECT (item), "cell");
-	description = gpm_cell_get_description (cell);
-	icon_name = gpm_cell_get_icon (cell);
-#else
 	object_path = g_object_get_data (G_OBJECT (item), "object-path");
 	egg_debug ("object_path=%s", object_path);
-	if (object_path == NULL)
+	if (object_path == NULL) {
+		dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
+						 GTK_MESSAGE_INFO, GTK_BUTTONS_OK, "%s",
+						 _("Device information"));
+		gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (dialog),
+							    "%s", _("There is no detailed information for this device"));
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (GTK_WIDGET (dialog));
 		goto out;
+	}
 
 	device = dkp_device_new ();
 	ret = dkp_device_set_object_path (device, object_path);
@@ -246,7 +217,6 @@ gpm_tray_icon_show_info_cb (GtkMenuItem *item, gpointer data)
 	obj = dkp_device_get_object (device);
 	icon_name = gpm_devicekit_get_object_icon (obj);
 	description = gpm_devicekit_get_object_description (obj);
-#endif
 
 	image = gtk_image_new ();
 	dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -263,10 +233,8 @@ gpm_tray_icon_show_info_cb (GtkMenuItem *item, gpointer data)
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 
 out:
-#ifdef HAVE_DK_POWER
 	if (device != NULL)
 		g_object_unref (device);
-#endif
 	g_free (description);
 	g_free (icon_name);
 }
@@ -509,53 +477,6 @@ gpm_tray_icon_popup_menu_cb (GtkStatusIcon *status_icon, guint button, guint32 t
 			  G_CALLBACK (gpm_tray_icon_popup_cleared_cd), icon);
 }
 
-#ifndef HAVE_DK_POWER
-/**
- * gpm_tray_icon_add_device:
- *
- * Add all the selected type of devices to the menu to form "drop down" info.
- **/
-static guint
-gpm_tray_icon_add_device (GpmTrayIcon *icon, GtkMenu *menu, GpmCellArray *array)
-{
-	GtkWidget *item;
-	GtkWidget *image;
-	GpmCell *cell;
-	GpmCellUnit *unit;
-	guint i;
-	gchar *icon_name;
-	gchar *label;
-	const gchar *desc;
-	guint len;
-
-	len = gpm_cell_array_get_num_cells (array);
-	for (i=0; i<len; i++) {
-		cell = gpm_cell_array_get_cell (array, i);
-		unit = gpm_cell_get_unit (cell);
-
-		egg_debug ("adding device '%i'", i);
-
-		/* generate the label */
-		desc = gpm_cell_unit_get_kind_localised (unit, FALSE);
-		label = g_strdup_printf ("%s (%.1f%%)", desc, unit->percentage);
-		item = gtk_image_menu_item_new_with_label (label);
-		g_free (label);
-
-		/* generate the image */
-		icon_name = gpm_cell_unit_get_icon (unit);
-		image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
-		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-		g_free (icon_name);
-
-		/* callback with the UDI and add the the menu */
-		g_signal_connect (G_OBJECT (item), "activate",
-				  G_CALLBACK (gpm_tray_icon_show_info_cb), icon);
-		g_object_set_data (G_OBJECT (item), "cell", (gpointer) cell);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	}
-	return len;
-}
-#else
 /**
  * gpm_tray_icon_add_device:
  **/
@@ -604,7 +525,6 @@ gpm_tray_icon_add_device (GpmTrayIcon *icon, GtkMenu *menu, const GPtrArray *arr
 	}
 	return added;
 }
-#endif
 
 /**
  * gpm_tray_icon_activate_cb:
@@ -620,19 +540,10 @@ gpm_tray_icon_activate_cb (GtkStatusIcon *status_icon, GpmTrayIcon *icon)
 	GtkWidget *item;
 	GtkWidget *image;
 	guint dev_cnt = 0;
-
+	const GPtrArray *array;
 	egg_debug ("icon left clicked");
 
 	/* add all device types to the drop down menu */
-#ifndef HAVE_DK_POWER
-	dev_cnt += gpm_tray_icon_add_device (icon, menu, icon->priv->collection->primary);
-	dev_cnt += gpm_tray_icon_add_device (icon, menu, icon->priv->collection->ups);
-	dev_cnt += gpm_tray_icon_add_device (icon, menu, icon->priv->collection->phone);
-	dev_cnt += gpm_tray_icon_add_device (icon, menu, icon->priv->collection->mouse);
-	dev_cnt += gpm_tray_icon_add_device (icon, menu, icon->priv->collection->keyboard);
-	dev_cnt += gpm_tray_icon_add_device (icon, menu, icon->priv->collection->pda);
-#else
-	const GPtrArray *array;
 	array = gpm_engine_get_devices (icon->priv->engine);
 	dev_cnt += gpm_tray_icon_add_device (icon, menu, array, DKP_DEVICE_TYPE_BATTERY);
 	dev_cnt += gpm_tray_icon_add_device (icon, menu, array, DKP_DEVICE_TYPE_UPS);
@@ -641,7 +552,7 @@ gpm_tray_icon_activate_cb (GtkStatusIcon *status_icon, GpmTrayIcon *icon)
 	dev_cnt += gpm_tray_icon_add_device (icon, menu, array, DKP_DEVICE_TYPE_PDA);
 	dev_cnt += gpm_tray_icon_add_device (icon, menu, array, DKP_DEVICE_TYPE_PHONE);
 	g_ptr_array_foreach ((GPtrArray*)array, (GFunc) g_object_unref, NULL);
-#endif
+
 	/* nothing to display! */
 	if (dev_cnt == 0 && !icon->priv->show_suspend && !icon->priv->show_hibernate)
 		return;
@@ -734,11 +645,8 @@ gpm_tray_icon_init (GpmTrayIcon *icon)
 
 	icon->priv = GPM_TRAY_ICON_GET_PRIVATE (icon);
 
-#ifndef HAVE_DK_POWER
-	icon->priv->collection = NULL;
-#else
 	icon->priv->engine = gpm_engine_new ();
-#endif
+
 	/* use libnotify */
 	icon->priv->notify = gpm_notify_new ();
 
@@ -796,9 +704,7 @@ gpm_tray_icon_finalize (GObject *object)
 		g_object_unref (tray_icon->priv->notify);
 	g_object_unref (tray_icon->priv->control);
 	g_object_unref (tray_icon->priv->status_icon);
-#ifndef HAVE_DK_POWER
 	g_object_unref (tray_icon->priv->engine);
-#endif
 	g_return_if_fail (tray_icon->priv != NULL);
 
 	G_OBJECT_CLASS (gpm_tray_icon_parent_class)->finalize (object);
