@@ -24,7 +24,6 @@
 #define __GPM_TRAY_ICON_H
 
 #include <glib-object.h>
-#include "gpm-engine.h"
 
 G_BEGIN_DECLS
 
@@ -57,8 +56,6 @@ gboolean	 gpm_tray_icon_set_tooltip		(GpmTrayIcon	*icon,
 							 const gchar	*tooltip);
 gboolean	 gpm_tray_icon_set_icon			(GpmTrayIcon	*icon,
 							 const gchar	*filename);
-gboolean	 gpm_tray_icon_set_collection_data	(GpmTrayIcon	*icon,
-							 GpmEngineCollection *collection);
 
 G_END_DECLS
 
